/* 
    Automated Drink Mixer

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2007 Jason Hunt
    nulluser@gmail.com
    
    File: database.cpp
*/

#include "core.h"
#include <mysql.h> 

drink_type *drinks;              // Structure of loaded drinks
unsigned int num_drinks = 0;     // Number of loaded drinks

int *channels;                   // Contains the ing_id in the channel
unsigned int num_channels = 0;
        
static MYSQL *mysql; 


/* Copy sql error to log file */
void database_error(char *msg)
{
    debug("sql_error", msg);

    char *e = (char *)mysql_error(mysql);

    if (e != NULL)
        debug("sql_error", e);
}
/* End of sql_error */


/* Load channel configuration from database */
void database_load_channels( void )
{
    num_channels = 0;

    debug("load_channels", "Loading channels");

    if (mysql_query(mysql, "select ing_id from channels order by chan_num"))
    {
        database_error ("Unable get mixing channels\n");        
        return;
    }
  
    MYSQL_RES *result = mysql_store_result(mysql);

    if (result == NULL) return;
   
    MYSQL_ROW row;
    
    // Number of mixers for this drink is the number of rows in result
    unsigned int num_load = mysql_num_rows(result);
    
    char tmp[400];
    sprintf(tmp, "Loading %d channel\n", num_load);
    debug("load_channels", tmp);
    
    // Get memory for channels
    channels = (int *) malloc(sizeof(int) * num_load);
    
    mixer_channel_stats = (channel_stat_type *) malloc(num_load * sizeof(channel_stat_type));
    
    while ((row = mysql_fetch_row(result)) && num_channels < num_load)
    {
        if (mysql_num_fields(result) != 1) break;
    
        // Store the ingredient id in the channel
        channels[num_channels++] = atoi(row[0]);
    }   
          
    mysql_free_result(result);  
}
/* End of load channels */


/* Figure out which channel an ingrdient is on */
bool database_get_channel(unsigned int ing_id,  int &chan_num)
{
    chan_num = NO_CHANNEL; // Indicator for not found
    
    for (int i = 0; i < num_channels; i++)
    {    
        if (channels[i] == ing_id)
        {
            chan_num = i+1; // Channels are numbered 1..NUM_CHANNELS
            return(false);
        }
    }
    
    return(true);
}
/* End of get_channel_number */


/* Load mixer data for a specific drink */
void database_load_mixers(unsigned int drink_num, unsigned int drink_id)
{
    drinks[drink_num].mixers_found = true;

    char query[1000];
    
    sprintf(query, "select ingredients.name, mixers.ing_id, mixers.amount, \
                    ingredients.cpo,  ingredients.alcohol_content, ingredients.pour_rate  \
                    from mixers,ingredients \
                    where mixers.ing_id = ingredients.ing_id and mixers.drink_id = '%d'", 
                    drink_id);

    if (mysql_query(mysql, query))
    {
        database_error ("Unable get mixers for drink\n");        
        return;
    }
  
    MYSQL_RES *result = mysql_store_result(mysql);

    if (result == NULL) return;
   
    MYSQL_ROW row;
    
    unsigned int loaded_mixers = 0;
    
    // Number of mixers for this drink is the number of rows in result
    unsigned int num_load = mysql_num_rows(result);
    
    char tmp[400];
    sprintf(tmp, "Loading %d mixers\n", num_load);
    debug("load_mixers", tmp);
    
    // Get memory for mixers
    drinks[drink_num].mixers = (mixer_type *) malloc(num_load * sizeof(mixer_type));
    
    // Load all mixers
    while ((row = mysql_fetch_row(result)) && drinks[drink_num].num_mixers < num_load)
    {
        if (mysql_num_fields(result) != 6) break;
      
        char ing_name[400];
              
        strcpy(ing_name, row[0]);

        unsigned int ing_id = atoi(row[1]);  // ingredient id

        float mix_amount = atof(row[2]);  // Mixer amount   
        float mix_cpo = atof(row[3]);  // Mixer amount   
        float mix_content = atof(row[4]);  // Mixer amount   
        float mix_rate = atof(row[5]);  // Mixer amount   
         
        
        // Get memory for mixer name 
        drinks[drink_num].mixers[loaded_mixers].name = (char *) malloc(strlen(ing_name)+1);
        
        // Copy ingredient name into mixer name
        strcpy(drinks[drink_num].mixers[loaded_mixers].name, ing_name);
        drinks[drink_num].mixers[loaded_mixers].volume = mix_amount;
        drinks[drink_num].mixers[loaded_mixers].cost = mix_cpo;
        drinks[drink_num].mixers[loaded_mixers].content = mix_content;
        drinks[drink_num].mixers[loaded_mixers].pour_rate = mix_rate;

        int chan_num = NO_CHANNEL;

        if (database_get_channel(ing_id, chan_num))
        {
            debug("load_mixers", "Unable to get channel number");
            drinks[drink_num].mixers_found = false;
        }

        drinks[drink_num].mixers[loaded_mixers].channel = chan_num;

        loaded_mixers++;

        char tmp[400];
        sprintf(tmp, "  %4.2f  %s    (chan: %d)\n", mix_amount, ing_name, chan_num);
        debug("load_mixers", tmp);
    }   
    
    drinks[drink_num].num_mixers = loaded_mixers;        
              
    mysql_free_result(result);           
        
}
/* End of load mixers */


/* Load all drinks from database */
void database_load_drinks( void )
{
    num_drinks = 0;    
    
    if (mysql_query(mysql, "select drink_id, name from drinks order by name"))
        database_error ("Unable get drink names\n");
  
    MYSQL_RES *result = mysql_store_result(mysql);
  
    if (result == NULL) 
    {
        database_error("No drinks in database");
        return;
    }
    
    MYSQL_ROW row;
    
    unsigned int num_load = mysql_num_rows(result);
    
    char t[400];
    sprintf(t, "Number of drinks in database: %d\n", num_load);
    debug("load_drinks", t);
    
    drinks = (drink_type *) malloc(num_load * sizeof(drink_type));
    
    while ((row = mysql_fetch_row(result)) && num_drinks < num_load)
    {
        int num_fields = mysql_num_fields(result);

        if (num_fields != 2 || row[0] == NULL || row[1] == NULL) break;

        char drink_name[400];
    
        strcpy(drink_name, row[1]);
        
        unsigned int drink_id = atoi(row[0]);
                   
        // Get memory for drink name
        drinks[num_drinks].name = (char *) malloc(strlen(drink_name)+1);
         
        strcpy(drinks[num_drinks].name, drink_name);
        
        drinks[num_drinks].num_mixers = 0;
               
        database_load_mixers(num_drinks, drink_id);                                    

        char tmp[400];
        sprintf(tmp, "Drink name:  %s  Drink ID: %d all_mixers  %d\n", 
                     drink_name, drink_id, drinks[num_drinks].mixers_found);
        debug("load_drinks", tmp);
                      
        num_drinks++;
    }
        
    mysql_free_result(result);  
}
/* End of load drinks */


/* Load the drink database from the mysql server */
void database_start( void )
{
    debug("load_database", "Loading data from mysql\n");

    mysql = mysql_init(NULL); 

    if(mysql == NULL) 
    {
        database_error("mysql_init failed");
        return;
    }

    if (mysql_real_connect(mysql, MYSQL_HOST, MYSQL_USER,  MYSQL_PASS, 
                           MYSQL_DBASE,  0, NULL, 0) == NULL)
    {                            
        database_error("mysql_real_connect failed");
        return;
    }
                
    database_load_channels();

    database_load_drinks();
  
    mysql_close(mysql);
}
/* End of load_database */


/* Free all memory for drink database */
void database_stop( void )
{
    for (int i = 0; i < num_drinks; i++)
    {
        // Free mixer names
        for (int j = 0; j < drinks[i].num_mixers; j++)
            free(drinks[i].mixers[j].name);   
            
        free(drinks[i].mixers); // free entire mixer structure       
          
        free(drinks[i].name); // free name data            
    }
    
    free(channels);
    free(mixer_channel_stats);
    
    // Free entire drink structure 
    free(drinks);
    
    debug("database_stop", "done");        
}
/* End of unload_database */


/* Reload the database */
void database_reload( void )
{
    // Do not reload the database if we are making a drink
    if (mixer_making_drink) return;

    database_stop();   
    database_start();
        
    InvalidateRect(main_hwnd, NULL, 0);
     
    display_set_mode(display_mode);
     
    sel_drink = 0;
    sel_base = 0;
    display_mode = DRINK_LIST;
}
/* End of database_reload */


